'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (content) {
    return tokenize(content);
};

var _input = require('./input');

var _input2 = _interopRequireDefault(_input);

var _logger = require('../logger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var token = {
    metadata: function metadata(key, value) {
        return {
            name: 'Metadata',
            key: key,
            value: value
        };
    },
    slideSeparator: function slideSeparator() {
        return {
            name: 'SlideSeparator'
        };
    },
    heading: function heading(level, value) {
        return {
            name: 'Heading',
            level: level,
            value: value
        };
    },
    orderedListItem: function orderedListItem(level, value) {
        return {
            name: 'OrderedListItem',
            level: level,
            value: value
        };
    },
    unorderedListItem: function unorderedListItem(level, value) {
        return {
            name: 'UnorderedListItem',
            level: level,
            value: value
        };
    },
    layoutBlockOpen: function layoutBlockOpen(block) {
        return {
            name: 'LayoutBlockOpen',
            block: block
        };
    },
    layoutBlockClose: function layoutBlockClose() {
        return {
            name: 'LayoutBlockOpen'
        };
    },
    specialBlock: function specialBlock(block, value, args) {
        return {
            name: 'SpecialBlock',
            block: block,
            value: value,
            args: args
        };
    },
    paragraph: function paragraph(value) {
        return {
            name: 'Paragraph',
            value: value
        };
    }
};

var REGEX_PLAIN_LINE = /^((?:[^:\-=.[*']|(?:\*(?![\s*]))).*)$/;
var REGEX_METADATA = /^:([^:]+):\s*(.*)$/;
var REGEX_SLIDE_SEPARATOR = /^'''$/;
var REGEX_HEADING = /^(={1,6})\s+(.*)$/;
var REGEX_ORDERED_LIST_ITEM = /^(\.{1,5})\s+(.*)$/;
var REGEX_UNORDERED_LIST_ITEM = /^(\*{1,5})\s+(.*)$/;
var REGEX_BLOCK_HEADER = /^\[(.+)\]$/;
var REGEX_LAYOUT_BLOCK_MARKER = /^--$/;
var REGEX_SPECIAL_BLOCK_MARKER = /^----$/;

function tokenize(content) {
    var input = new _input2.default(content);
    var tokens = [];

    while (!input.eof()) {
        var line = input.nextLine();

        if (line === '') {
            continue;
        } else if (REGEX_METADATA.test(line)) {
            var match = REGEX_METADATA.exec(line);
            tokens.push(token.metadata(match[1], match[2]));
        } else if (REGEX_SLIDE_SEPARATOR.test(line)) {
            tokens.push(token.slideSeparator());
        } else if (REGEX_HEADING.test(line)) {
            var _match = REGEX_HEADING.exec(line);
            tokens.push(token.heading(_match[1].length, _match[2]));
        } else if (REGEX_ORDERED_LIST_ITEM.test(line)) {
            var _match2 = REGEX_ORDERED_LIST_ITEM.exec(line);
            var _level = Number(_match2[1].length);
            var _value = _match2[2] + eatPlainLines(input);
            tokens.push(token.orderedListItem(_level, _value));
        } else if (REGEX_UNORDERED_LIST_ITEM.test(line)) {
            var _match3 = REGEX_UNORDERED_LIST_ITEM.exec(line);
            var _level2 = Number(_match3[1].length);
            var _value2 = _match3[2] + eatPlainLines(input);
            tokens.push(token.unorderedListItem(_level2, _value2));
        } else if (REGEX_BLOCK_HEADER.test(line)) {
            var _match4 = REGEX_BLOCK_HEADER.exec(line);
            line = input.nextLine();
            if (REGEX_LAYOUT_BLOCK_MARKER.test(line)) {
                tokens.push(token.layoutBlockOpen(_match4[1]));
            } else if (REGEX_SPECIAL_BLOCK_MARKER.test(line)) {
                var _value3 = '';
                while (!REGEX_SPECIAL_BLOCK_MARKER.test(line = input.nextLine()) && !input.eof()) {
                    _value3 += line + '\n';
                }

                var _args = _match4[1].split(',').map(function (arg) {
                    return arg.trim();
                });
                tokens.push(token.specialBlock(_args[0], _value3, _args.slice(1)));
            } else {
                (0, _logger.error)('syntax error on line ' + input.current() + ': block header requires a block opening marker');
            }
        } else if (REGEX_LAYOUT_BLOCK_MARKER.test(line)) {
            tokens.push(token.layoutBlockClose());
        } else if (REGEX_PLAIN_LINE.test(line)) {
            var _value4 = line + eatPlainLines(input);
            tokens.push(token.paragraph(_value4));
        } else {
            (0, _logger.warn)(_logger.IMPLEMENTATION_ERROR_MESSAGE);
        }
    }

    return tokens;
}

function eatPlainLines(input) {
    var value = ' ';
    do {
        var line = input.peekLine();
        if (REGEX_PLAIN_LINE.test(line)) {
            line = input.nextLine();
            value += ' ' + line;
        } else {
            return value === ' ' ? '' : value;
        }
    } while (!input.eof());

    return value;
}